package betterwithmods.module.tweaks;

import betterwithmods.common.BWMBlocks;
import betterwithmods.common.BWMItems;
import betterwithmods.common.blocks.BlockUnfiredPottery;
import betterwithmods.common.items.ItemMaterial;
import betterwithmods.common.registry.blockmeta.managers.KilnManager;
import betterwithmods.common.registry.blockmeta.recipe.KilnRecipe;
import betterwithmods.module.Feature;
import com.google.common.collect.Lists;
import net.minecraft.block.*;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumActionResult;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.event.entity.player.PlayerInteractEvent;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;

public class PlaceClay extends Feature {


    @Override
    public String getFeatureDescription() {
        return "Clay and Nether Sludge are placeable blocks.";
    }

    @Override
    public void init(FMLInitializationEvent event) {
        KilnManager.INSTANCE.addRecipe(new KilnRecipe(BWMBlocks.UNFIRED_POTTERY, BlockUnfiredPottery.EnumType.BRICK.getMeta(), Lists.newArrayList(new ItemStack(Items.field_151118_aC))) {
            @Override
            public ItemStack getStack() {
                return new ItemStack(Items.field_151119_aD);
            }
        });
        KilnManager.INSTANCE.addRecipe(new KilnRecipe(BWMBlocks.UNFIRED_POTTERY, BlockUnfiredPottery.EnumType.NETHER_BRICK.getMeta(), Lists.newArrayList(new ItemStack(Items.field_151130_bT))) {
            @Override
            public ItemStack getStack() {
                return ItemMaterial.getMaterial(ItemMaterial.EnumMaterial.NETHER_SLUDGE);
            }
        });
    }

    @SubscribeEvent
    public void onPlaceClay(PlayerInteractEvent.RightClickBlock event) {
        if (event.isCanceled())
            return;

        World worldIn = event.getWorld();
        EntityPlayer player = event.getEntityPlayer();
        EnumFacing facing = event.getFace();
        BlockPos pos = event.getPos();
        EnumHand hand = event.getHand();

        IBlockState iblockstate = worldIn.func_180495_p(pos);
        Block block = iblockstate.func_177230_c();

        if (canPlaceAt(player, worldIn, pos)) {
            event.setCancellationResult(EnumActionResult.FAIL);
            return;
        }
        if (!block.func_176200_f(worldIn, pos)) {
            pos = pos.func_177972_a(facing);
        }


        ItemStack itemstack = player.func_184586_b(hand);
        IBlockState stateToPlace = null;

        if (itemstack.func_190926_b())
            return;
        if (itemstack.func_77973_b() == Items.field_151119_aD)
            stateToPlace = BWMBlocks.UNFIRED_POTTERY.func_176223_P().func_177226_a(BlockUnfiredPottery.TYPE, BlockUnfiredPottery.EnumType.BRICK);
        if (itemstack.func_77973_b() == BWMItems.MATERIAL && itemstack.func_77960_j() == ItemMaterial.EnumMaterial.NETHER_SLUDGE.ordinal())
            stateToPlace = BWMBlocks.UNFIRED_POTTERY.func_176223_P().func_177226_a(BlockUnfiredPottery.TYPE, BlockUnfiredPottery.EnumType.NETHER_BRICK);

        if (stateToPlace != null && player.func_175151_a(pos, facing, itemstack) && worldIn.func_190527_a(stateToPlace.func_177230_c(), pos, false, facing, null)) {
            if (placeBlockAt(itemstack, player, worldIn, pos, stateToPlace)) {
                SoundType soundtype = worldIn.func_180495_p(pos).func_177230_c().getSoundType(worldIn.func_180495_p(pos), worldIn, pos, player);
                worldIn.func_184133_a(player, pos, soundtype.func_185841_e(), SoundCategory.BLOCKS, (soundtype.func_185843_a() + 1.0F) / 2.0F, soundtype.func_185847_b() * 0.8F);
                if (!player.func_184812_l_())
                    itemstack.func_190918_g(1);
            }

            event.setCancellationResult(EnumActionResult.SUCCESS);
            event.setCanceled(true);
        }
    }

    private boolean placeBlockAt(ItemStack stack, EntityPlayer player, World world, BlockPos pos, IBlockState newState) {
        if (!world.func_180501_a(pos, newState, 11)) return false;

        IBlockState state = world.func_180495_p(pos);
        if (state.func_177230_c() == newState.func_177230_c()) {
            newState.func_177230_c().func_180633_a(world, pos, state, player, stack);
        }

        return true;
    }


    private boolean canPlaceAt(EntityPlayer player, World world, BlockPos pos) {
        if (player.func_70093_af())
            return true;
        if (world.func_175625_s(pos) != null)
            return false;
        IBlockState state = world.func_180495_p(pos);
        Block block = state.func_177230_c();
        return !(block instanceof BlockWorkbench || block instanceof BlockDoor || block instanceof BlockTrapDoor || block instanceof BlockFenceGate);
    }


    @Override
    public boolean hasSubscriptions() {
        return true;
    }
}
